@@ -127,7 +127,7 @@ class TranslationSolvers():
             pass
 
 
-    def solve_ballistic_model(self, ki,ke, poi=None, tag= None):
+    def solve_ballistic_model(self, ki,ke, nt_seq, length, poi=None, tag= None):
         '''
 
         Given a iniation rate and set of elongation rates,
@@ -1391,7 +1391,8 @@ class TranslationSolvers():
 
         probe_vec = ssa_conditions['probe_vec']
 
-        colors = self.colors
+        colors = int(probe_vec.shape[0])
+        print(probe_vec.shape)
 
 
         if ssa_conditions['bursting'] == False:
@@ -1445,7 +1446,7 @@ class TranslationSolvers():
                 intime = float(intime)
                 seed = seeds[i]
                 colors = int(colors)
-
+            print(colors)
             ssa_translation_lowmem.run_SSA(result, ribtimes, coltimes, colpointsx,colpointst, kelong,frapresult, t, ki, kt, evf, evi, intime, seed,nribs,x0,footprint, probe_vec ,colors, kon, koff, kprobe, probe_loc, flags, N_rib)
             #ssa_translation.run_SSA(result, ribtimes, coltimes, k[1:-1],frapresult, truetime, k[0], k[-1], evf, evi, intime, seeds[i],nribs)
 
